/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Core/Message/Message.h>

#include <QFile>
#include <QObject>

#include "MavlinkVersionHeader.h"
#include "Ports/MavlinkPort.h"

class BaseMavlinkModule;

class MavlinkCodec : public QObject
{
    Q_OBJECT
public:
    enum SysId
    {
        SysIdMain       = MAV_SYSID_MAIN,
        SysIdPayload    = MAV_SYSID_PAYLOAD,
        SysIdRig        = MAV_SYSID_RIG,
        SysIdGsReceiver = MAV_SYSID_GS,
    };

    enum CompId
    {
        CompIdNone = 0
    };

    explicit MavlinkCodec(MavlinkPort* port, BaseMavlinkModule* parent);

    void send(const mavlink_message_t& msg);

    void startReading();
    void stopReading();

    bool encodeMessage(const Message& msg, SysId sysId, CompId compId,
                       mavlink_message_t& output);
    Message decodeMessage(const mavlink_message_t& msg);

    void closeLog();
    void openLogFile();

    static const mavlink_message_info_t& getMessageFormat(uint8_t messageId);

signals:
    void msgReceived(const Message& msg);

private slots:
    void onBytesReceived(const char* data, qsizetype len);

private:
    /* convert a field of any type to a string */
    Field decodeField(const mavlink_message_t& msg,
                      const mavlink_field_info_t& field);

    /* if field is an array, decode every element in the array */
    Field decodeArrayElement(const mavlink_message_t& msg,
                             const mavlink_field_info_t& field, int idx);

    void setLogFilePath();

    mavlink_message_t mavlinkMessage;
    mavlink_status_t mavlinkStatus;

    MavlinkPort* port;
    BaseMavlinkModule* parent;

    QString logFilePath = "";
    QFile logFile;

    QMap<QString, int> msgNameIdMap;
};