/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Core/Message/Message.h>
#include <Modules/Module.h>

#include <QDateTime>
#include <QList>
#include <QPair>
#include <QTimer>
#include <QWidget>

class OutgoingMessagesViewerModule : public Module
{
    Q_OBJECT

public:
    OutgoingMessagesViewerModule();
    ~OutgoingMessagesViewerModule();

    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject& xmlObject) override;

private:
    void setupUi();
    void customContextMenuActionSetup();

    void resizeEvent(QResizeEvent* event = nullptr) override;

    void handleMsg(const Message& msg);
    void handleAck(const Message& ack);
    void handleNack(const Message& nack);
    void handleWack(const Message& wack);

    void addMessageToTable(const QPair<QDateTime, Message>& msg, int row = 0);
    void clearTable();

    void updateElapsedTime();

    QTableWidget* table;
    QTimer* timer;
    QAction* timestamp;
    QAction* elapsedTime;

    QList<QPair<QDateTime, Message>> messages;

    int maxListSize = 20;
    int timerPeriod = 1000;  // [ms]
};
