/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Modules/Module.h>

#include <QTime>
#include <QTimer>

namespace Ui
{
class TimerControllerModule;
}

class TimerControllerModule : public Module
{
    Q_OBJECT

public:
    TimerControllerModule();
    ~TimerControllerModule();

protected:
    void onSetTimerClicked();
    void onStartTimerClicked();
    void onHoldTimerClicked();
    void onSetInfoLabelClicked();
    void onTimerTick();
    void onEnableOutputToggled(bool val);

    QString updateTimeLabel();
    void setInfoLabel(const QString &txt);
    void updateOutputFile(const QString &outputTxt) const;

    void initCurrentTime();
    QString getPrefix() const;
    bool isTimeExpired() const;
    int getIncrement() const;

private:
    Ui::TimerControllerModule *ui;
    QTimer timer;
    QString timeFormat = "hh:mm:ss";
    QTime currentTime;
    int timerIntervalMs = 1000;
    int increment       = -1;
    bool isHold;
    bool isOutputEnabled = false;
    QString holdTxt      = "HOLD";

    QString tMinus = "T- ";
    QString tPlus  = "T+ ";
};
